<?php

class BifitKassa
{
	public function __construct()
	{
	}

	protected function getPayment($orderId)
	{
		$dbRes = \Bitrix\Sale\PaymentCollection::getList(array(
			'select' => array('*'),
			'filter' => array(
				'=ORDER_ID' => $orderId
			)
		));

		$sumCash = 0;
		$sumEcash = 0;

		while ($item = $dbRes->fetch()) {
			$paySystemId = $item['PAY_SYSTEM_ID'];
			$arFilter = array(
				'PAY_SYSTEM_ID' => $paySystemId
			);
			$queryPaymentSystem = CSalePaySystemAction::GetList(
				array(),
				$arFilter,
				false,
				false,
				['*']
			);
			$paymentSystem = $queryPaymentSystem->Fetch();

			if ($paymentSystem['IS_CASH'] == 'Y') {
				$sumCash += $item['SUM'];
			} else {
				$sumEcash += $item['SUM'];
			}
		}

		return array(
			'cash' => $sumCash,
			'ecash' => $sumEcash
		);
	}

	function __formatprc($price)
	{
		CModule::IncludeModule("sale");

		$decimals = 2;
		$formatInfo = CCurrencyLang::GetCurrencyFormat(
			COption::GetOptionString("sale", "default_currency", "RUB")
		);

		if (isset($formatInfo['DECIMALS'])) {
			$decimals = intval($formatInfo['DECIMALS']);
		}

		$dec_count = $decimals;

		// форматируем цену согласно настроек
		$price =
			$dec_count < 0
				? round($price, $dec_count)
				: number_format($price, $dec_count, '.', '');

		return $price;
	}

	function __checkInn($inn, $type = 1)
	{
		// проверяем заполненность
		if (isset($inn) && trim($inn) != '') {
			if ($type == 2) {
				// только цифры в количестве 10 шт.
				$inn = !preg_match("/^\d{10}$/", trim($inn))
					? GetMessage('BIFIT_WRONG_INN')
					: trim($inn);
			} elseif ($type == 1) {
				// только цифры в количестве 12 шт.
				$inn = !preg_match("/^\d{12}$/", trim($inn))
					? GetMessage('BIFIT_WRONG_INN')
					: trim($inn);
			} else {
				// только цифры в количестве 10 или 12 шт.
				$inn = !preg_match("/^\d{10}$|^\d{12}$/", trim($inn))
					? GetMessage('BIFIT_WRONG_INN')
					: trim($inn);
			}
		} else {
			// ИНН пуст или не найден!
			$inn = GetMessage('BIFIT_EMPTY_INN');
		}

		return $inn;
	}

	function __checkPhone($tel)
	{
		// проверяем заполненность
		if (isset($tel) && trim($tel) != '') {
			// удаляем лишние символы
			$phone = str_replace(
				array('+', '(', ')', '-', ' '),
				'',
				trim($tel)
			);

			// получаем первую цифру
			$first = substr($phone, "0", 1);

			// если номер из 10 цифр и первая цифра не 7, то добавляем 7
			$phone =
				strlen($phone) == 10 && $first != 7 ? '7' . $phone : $phone;

			// окончательная проверка
			$phone = !preg_match("/^[0-9]{11}+$/", $phone)
				? GetMessage('BIFIT_WRONG_PHONE')
				: $phone;
		} else {
			// Телефон пуст или не найден!
			$phone = GetMessage('BIFIT_EMPTY_PHONE');
		}

		return $phone;
	}

	function __checkEmail($email)
	{
		// проверка модели записи
		$email = !preg_match("/^\+\d+$|^\S+@\S+$/", $email)
			? GetMessage('BIFIT_WRONG_EMAIL')
			: $email;

		return $email;
	}

	function __checkProductPrice(
		$item_price,
		$order_subtotal,
		$order_discount = 0
	) {
		// если скидка больше нуля
		if ($order_discount > 0) {
			// вычисляем процент скидки
			$percent = ($order_discount * 100) / $order_subtotal;

			// уменьшаем цену товара согласно скидки
			$item_price = $item_price - $item_price * ($percent / 100);
		}

		return $item_price;
	}

	function __loadData($url, $headers = false, $data = false, $auth = [])
	{
		// инициализируем сеанс cURL
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);

		// добавляем данные
		if ($data) {
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
		}

		// возврат результата в качестве строки
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

		// добавляем headers
		if ($headers) {
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		}

		// добавляем авторизацию
		if (count($auth) == 2) {
			$user = $auth['user'];
			$pass = $auth['pass'];
			curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
			curl_setopt($ch, CURLOPT_USERPWD, "$user:$pass");
		}

		// получаем ответ
		$output = curl_exec($ch);

		// завершаем сеанс cURL
		curl_close($ch);

		return $output;
	}

	function __saveReport($test, $bifit, $receipt, $receipt_id, $doc, $logObj)
	{
		// выводим свойство операции
		$log =
			$test != ''
				? '####### ' . GetMessage('BIFIT_TEST_REQUEST') . "\n"
				: '####### ' . GetMessage('BIFIT_BATTLE_REQUEST') . "\n";

		// устанавливаем часовой пояс согласно настройкам сайта
		$log .= '####### ' . date('d-m-Y H:i:s', time()) . "\n";

		// ОТВЕТ #1

		$log .= '####### ' . GetMessage('BIFIT_ANSWER') . ' #1: ' . "\n";

		// данные ответа #1
		if (is_object($bifit)) {
			foreach ($bifit as $key => $val) {
				$log .= $key . ': ' . $val . "\n";
			}
		}

		// ЧЕК

		// сформированный чек для отправки
		$log .= '####### ' . GetMessage('BIFIT_CHECK') . ': ' . "\n" . $receipt . "\n";

		// ОТВЕТ #2

		$log .= '####### ' . GetMessage('BIFIT_ANSWER') . ' #2: ' . "\n";

		// данные ответа #2
		if (is_object($receipt_id)) {
			$log .= print_r($receipt_id, true) . "\n";
		} else {
			$log .= $receipt_id . "\n";

			// ОТВЕТ #3

			$log .= '####### ' . GetMessage('BIFIT_ANSWER') . ' #3: ' . "\n";
			$log .= is_object($doc) ? print_r($bifit, true) : '';

			// ССЫЛКА НА ЧЕК

			// сформированная ссылка на чек
			$log .= '####### ' . GetMessage('BIFIT_LINK_TO_CHECK') . ': ' . "\n" . $logObj->receipt_link;
		}

		$logObj->log = $log;

		// СОХРАНЯЕМ В БД

		global $DB;

		$connection = Bitrix\Main\Application::getConnection();
		$sqlHelper = $connection->getSqlHelper();
		$connection = \Bitrix\Main\Application::getConnection();
		$connection->queryExecute(
			"INSERT INTO b_bifit_kassa_reports (
			ORDER_ID, 
			ORDER_NUMBER,
			ORDER_DATE,
			ACTION_DATE,
			RECEIPT_LINK,
			RECEIPT_ID,
			RECEIPT_TYPE,
			LOG
			) VALUES ('" .
				$sqlHelper->forSql($logObj->order_id) .
				"', 
			'" .
				$sqlHelper->forSql($logObj->order_number) .
				"', 
			'" .
				$sqlHelper->forSql($logObj->order_date) .
				"', 
			'" .
				$sqlHelper->forSql($logObj->action_date) .
				"', 
			'" .
				$sqlHelper->forSql($logObj->receipt_link) .
				"', 
			'" .
				$sqlHelper->forSql($logObj->receipt_id) .
				"', 
			'" .
				$sqlHelper->forSql($logObj->receipt_type) .
				"',
			'" .
				$sqlHelper->forSql($logObj->log) .
				"'
			)"
		);
	}

	function __mainProcess($order, $test, $token, $i)
	{
		Bitrix\Main\Localization\Loc::loadMessages(__FILE__);

		global $DB;

		// 1. АВТОРИЗАЦИЯ
		//    процесс необходимый для подключения клиента к фискальному процессингу

		// адрес для POST-запроса
		$url = 'https://fp' . $test . '.bifit.com/processing-api/oauth/token';

		// данные ссылки
		$data = 'token=' . $token;
		$data .= '&client_id=processing-connector-token';
		$data .= '&client_secret=processing-connector-token';
		$data .= '&grant_type=token';

		// headers
		$headers = array('Content-Type: application/x-www-form-urlencoded');

		// декодируем полученный ответ
		$bifit = json_decode(self::__loadData($url, $headers, $data, []));

		// 2. ФОРМИРОВАНИЕ КОНТЕНТА ЧЕКА
		//    формирование информации для дальнейшей отправки на фискализацию

		// Параметры плагина

		// имя кассира
		$cashier_name = COption::GetOptionString('bifit.kassa', 'cashier_name')
			? COption::GetOptionString('bifit.kassa', 'cashier_name')
			: 'Иванов И.И.';

		// ИНН кассира
		$cashier_inn = COption::GetOptionString('bifit.kassa', 'cashier_inn')
			? COption::GetOptionString('bifit.kassa', 'cashier_inn')
			: '123456789012';

		// тип документа
		$arr_rec_type = array(
			'1' => 'SALE',
			'2' => 'SALE_RETURN',
			'3' => 'PURCHASE',
			'4' => 'PURCHASE_RETURN'
		);
		$receipt_type = $arr_rec_type[$i];

		// система налогов
		$tax_system = COption::GetOptionString('bifit.kassa', "tax_system_{$i}")
			? COption::GetOptionString('bifit.kassa', "tax_system_{$i}")
			: 'SIMPLIFIED_WITH_EXPENSE';

		// признак способа расчёта
		$calc_method = COption::GetOptionString(
			'bifit.kassa',
			'calculation_method'
		)
			? COption::GetOptionString('bifit.kassa', 'calculation_method')
			: 'FULL_PAY';

		// единый НДС
		$vat_mode = COption::GetOptionString('bifit.kassa', 'vat_mode')
			? COption::GetOptionString('bifit.kassa', 'vat_mode')
			: 1;

		// НДС
		$vat = COption::GetOptionString('bifit.kassa', 'vat')
			? COption::GetOptionString('bifit.kassa', 'vat')
			: 'WITHOUT_VAT';

		// тип оплаты
		$payment_type = COption::GetOptionString('bifit.kassa', 'payment_type')
			? COption::GetOptionString('bifit.kassa', 'payment_type')
			: 'CARD';

		// место расчётов
		$payment_address = COption::GetOptionString(
			'bifit.kassa',
			'payment_address'
		)
			? COption::GetOptionString('bifit.kassa', 'payment_address')
			: 'https://site.ru';

		// тип адреса клиента
		$address_type = COption::GetOptionString('bifit.kassa', 'address_type')
			? COption::GetOptionString('bifit.kassa', 'address_type')
			: 'email';

		// ФИО клиента
		$fio = COption::GetOptionString('bifit.kassa', 'fio')
			? COption::GetOptionString('bifit.kassa', 'fio')
			: 0;

		// пользователь
		$user = CUSer::GetByID($order['USER_ID'])->Fetch();
		$user_phone = 0;

		// данные заказа
		$order_id = (int) $order['ID'];
		$order_number = $order['ACCOUNT_NUMBER'];

		$dbOrderProps = CSaleOrderPropsValue::GetList(
			array(
				"SORT" => "ASC"
			),
			array(
				"ORDER_ID" => $order_id,
				"CODE" => array("PHONE")
			)
		);

		if ($arOrderProps = $dbOrderProps->Fetch()) {
			$user_phone = $arOrderProps['VALUE'];
		}

		// адрес клиента в чеке
		$address =
			$address_type == 'phone' && $user
				? self::__checkPhone($user_phone)
				: self::__checkEmail($user['EMAIL']);

		// позиции оплаты заказа
		$o_shipment = $order['PRICE_DELIVERY'];

		// параметры массива
		$o_subtotal = 0;
		$r_products = array();

		$dbBasket = CSaleBasket::GetList(
			array(
				"ID" => "ASC"
			),
			array(
				"ORDER_ID" => $order_id
			),
			false,
			false
		);

		$key_id = 0;

		while ($product = $dbBasket->GetNext()) {
			//// корректируем цену товара с учётом скидок
			$p_price = self::__formatprc(
				self::__checkProductPrice(
					$product['PRICE'],
					$order['PRICE'] + $order['TAX_VALUE'],
					abs($order['DISCOUNT_VALUE'])
				)
			);
			$o_subtotal += $p_price * $product['QUANTITY'];

			// очищаем имя товара от спец-символов
			$p_name = preg_replace(
				'/[^\p{L}0-9 ]/iu',
				' ',
				html_entity_decode($product['NAME'])
			);
			$p_name = preg_replace('/ +/', ' ', trim($p_name));

			// формируем массив товаров для чека
			$r_products[$key_id] = new stdClass();
			$r_products[$key_id]->product_id = $product['PRODUCT_ID'];
			$r_products[$key_id]->product_name = $p_name;
			$r_products[$key_id]->product_item_price = $p_price;
			$r_products[$key_id]->product_quantity = $product['QUANTITY'];
			$r_products[$key_id]->product_tax = 0;

			$key_id++;
		}

		// сверка итоговых сумм заказа и чека
		$total_1 = self::__formatprc($o_subtotal + $o_shipment);
		$total_2 = self::__formatprc($order['PRICE']);
		$diff =
			$total_1 > 0 && $total_2 > 0 && $total_1 != $total_2
				? abs($total_1 - $total_2)
				: 0;

		// параметры для чека
		$items = '';

		// формируем данные чека
		foreach ($r_products as $key_id => $product) {
			// цена товара
			$p_price = $product->product_item_price;

			// изменение цены товара согласно сверки
			if ($diff > 0 && $p_price > $diff) {
				$p_price =
					$total_2 > $total_1
						? $p_price + $diff / $product->product_quantity
						: $p_price - $diff / $product->product_quantity;
				$diff = 0;
			}

			// выводим данные товара в чек
			$items .= $items != '' ? ', ' : '';
			$items .= '{';
			$items .= '"calculationMethod":"' . $calc_method . '", ';
			$items .= '"paymentSubject":"PRODUCT", ';
			$items .= '"name":"' . $product->product_name . '", ';
			$items .= '"price":' . self::__formatprc($p_price) . ', ';
			$items .= '"quantity":' . $product->product_quantity . ', ';
			$items .=
				$vat_mode == 1
					? '"vat":"' . $vat . '", '
					: '"vat":"VAT_' . floor($product->product_tax) . '", ';
			$items .=
				'"total":' .
				self::__formatprc($p_price * $product->product_quantity);
			$items .= '}';
		}

		// доставка
		if ($o_shipment > 0) {
			$shipment_tax_percent = 0;

			// выводим данные способа доставки в чек
			$items .= $items != '' ? ', ' : '';
			$items .= '{';
			$items .= '"calculationMethod":"' . $calc_method . '", ';
			$items .= '"paymentSubject":"SERVICE", ';
			$items .= '"name":"Доставка", ';
			$items .= '"price":' . self::__formatprc($o_shipment) . ', ';
			$items .= '"quantity":1, ';
			$items .=
				$vat_mode == 1
					? '"vat":"' . $vat . '", '
					: '"vat":"VAT_' . floor($shipment_tax_percent) . '", ';
			$items .= '"total":' . self::__formatprc($o_shipment);
			$items .= '}';
		}

		// формируем чек
		$receipt = '{';
		$receipt .=
			'"type":"' .
			$receipt_type .
			'", "taxSystem":"' .
			$tax_system .
			'", ';
		$receipt .= '"cashier":{';
		$receipt .= '"name":"' . $cashier_name . '", ';
		$receipt .= '"inn":"' . self::__checkInn($cashier_inn, 1) . '"';
		$receipt .= '}, ';
		$receipt .= '"client":{';
		$receipt .= $fio
			? '"name":"' .
				trim($user['LAST_NAME'] . ' ' . $user['NAME']) .
				'", '
			: '';
		$receipt .= '"address":"' . $address . '"';
		$receipt .= '}, ';
		$receipt .= '"paymentAddress":"' . $payment_address . '", ';
		$receipt .= '"items":[' . $items . '], ';
		$receipt .= '"total":' . self::__formatprc($order['PRICE']) . ', ';
		$receipt .=
			'"payments":{"' .
			$payment_type .
			'": ' .
			self::__formatprc($order['PRICE']) .
			'}';
		$receipt .= '}';

		// 3. ОТПРАВКА ЧЕКА В ФИСКАЛЬНЫЙ ПРОЦЕССИНГ
		//    передача контента чека в фискальный процессинг для дальнейшей фискализации

		// адрес для POST-запроса
		$url =
			'https://fp' .
			$test .
			'.bifit.com/processing-api/protected/documents/registration/receipts';

		// данные чека
		$data = $receipt;

		// ключ идемпотентности (Idempotency-Key)
		$o_number = $order_number;
		$o_summ = self::__formatprc($order['PRICE']);
		$date_time = $order['DATE_INSERT'];
		$idemp_key = hash('sha256', "$o_number/$o_summ/$date_time");

		// headers
		$headers = array(
			'Content-Type: application/json',
			'Authorization: Bearer ' . $bifit->access_token,
			'Idempotency-Key: ' . $idemp_key
		);

		// получаем идентификатор документа на процессинге (id)
		$receipt_id = json_decode(self::__loadData($url, $headers, $data, []));

		// 4. ПОЛУЧЕНИЕ ДОКУМЕНТА ПО ID
		//    получение документа с фискальными признаками по его номеру

		// параметры
		$doc = '';

		if (!is_object($receipt_id)) {
			// ссылка для GET-запроса
			$url =
				'https://fp' .
				$test .
				'.bifit.com/processing-api/protected/documents/' .
				$receipt_id;

			// headers
			$headers = array('Authorization: Bearer ' . $bifit->access_token);

			// получаем документ
			$doc = json_decode(self::__loadData($url, $headers, false, []));
		}

		// ССЫЛКА НА ЧЕК

		$receipt_link = !is_object($receipt_id)
			? 'https://fp' .
				$test .
				'.bifit.com/processing-api/receipts/' .
				$idemp_key
			: 'XXX';

		// ЛОГ ОПЕРАЦИИ

		// параметры для лога
		$logObj = new stdClass();
		$logObj->order_id = $order_id;
		$logObj->order_number = $order_number;
		$logObj->order_date = $order['DATE_INSERT'];
		$logObj->action_date = date('Y-m-d H:i:s');
		$logObj->receipt_link = $receipt_link;
		$logObj->receipt_id = !is_object($receipt_id) ? $receipt_id : '000';
		$logObj->receipt_type = $receipt_type;
		$logObj->log = '';

		// сохраняем лог операции
		self::__saveReport($test, $bifit, $receipt, $receipt_id, $doc, $logObj);
	}

	public static function handleSaleOrderStatus($id, $val)
	{
		if (gettype($id) == 'object' || !CModule::IncludeModule('sale')) {
			return;
		}

		$helper = new BifitKassa();
		$order = CSaleOrder::GetByID($id);

		// данные заказа
		$status_id = $order['STATUS_ID'];
		$payment_id = $order['PAY_SYSTEM_ID'];
		$payed = $order['PAYED'];

		// параметры
		$test = COption::GetOptionString('bifit.kassa', 'test') ? '-test' : '';
		$token =
			$test != ''
				? 'P5cKbUUD9uSSrSlGdzspLblvBnD0GzTAE0cLmAPSEMxJ79DtLE'
				: COption::GetOptionString('bifit.kassa', 'token');

		for ($i = 1; $i <= 4; $i++) {
			// узнаём нужно ли использовать данный тип документа
			$use_type = COption::GetOptionString('bifit.kassa', "use_type_{$i}")
				? true
				: false;

			// Если использовать нужно
			if ($use_type) {
				// получаем параметры
				$order_paid = COption::GetOptionString(
					'bifit.kassa',
					"order_paid_{$i}"
				); // заказ оплачен (да/нет/не учитывать)(2/1/0)
				$pm_statuses = json_decode(
					COption::GetOptionString('bifit.kassa', "pm_statuses_{$i}")
				);
				$pm_ids = json_decode(
					COption::GetOptionString('bifit.kassa', "pm_ids_{$i}")
				);

				// Если совпадает способ оплаты и статус заказа
				if (
					isset($pm_ids) &&
					isset($pm_statuses) &&
					in_array($payment_id, $pm_ids) &&
					in_array($status_id, $pm_statuses) &&
					$token != ''
				) {
					// если выбрано "заказ оплачен",
					// а он НЕ оплачен, то проверяем следующий таб
					if ($order_paid == '2' && $payed != 'Y') {
						continue;
					}

					// если выбрано "заказ не оплачен",
					// а он оплачен, то проверяем следующий таб
					if ($order_paid == '1' && $payed != 'N') {
						continue;
					}

					// запускаем БИФИТ-процесс
					self::__mainProcess($order, $test, $token, $i);
				}
			}
		}
	}
}