<?php

$moduleId = 'bifit.kassa';

use Bitrix\Main\Loader;
use Bitrix\Main\Localization\Loc;

function AddMultiSelectField(
	$form,
	$id,
	$content,
	$required,
	$arSelect,
	$value = false,
	$arParams = array()
) {
	if ($value === false || $value == null) {
		$value = [];
	}

	$html = '<select name="' . $id . '" multiple';

	foreach ($arParams as $param) {
		$html .= ' ' . $param;
	}

	$html .= '>';

	foreach ($arSelect as $key => $val) {
		$html .=
			'<option value="' .
			htmlspecialcharsbx($key) .
			'"' .
			(in_array($key, $value) ? ' selected' : '') .
			'>' .
			htmlspecialcharsex($val) .
			'</option>';
	}

	$html .= '</select>';

	$form->tabs[$form->tabIndex]["FIELDS"][$id] = array(
		"id" => $id,
		"required" => $required,
		"content" => $content,
		"html" =>
			'<td width="40%">' .
			($required
				? '<span class="adm-required-field">' .
					$form->GetCustomLabelHTML($id, $content) .
					'</span>'
				: $form->GetCustomLabelHTML($id, $content)) .
			'</td><td>' .
			$html .
			'</td>',
		"hidden" =>
			'<input type="hidden" name="' .
			$id .
			'" value="' .
			htmlspecialcharsbx(join(', ', $value)) .
			'">'
	);
}

function GetStatuses()
{
	$arStatuses = array(
		"" => GetMessage("SMO_STATUS")
	);

	$dbStatus = CSaleStatus::GetList(
		array(
			"SORT" => "ASC"
		),
		array(
			"LID" => LANGUAGE_ID
		),
		false,
		false,
		array("ID", "NAME", "SORT")
	);

	while ($arStatus = $dbStatus->GetNext()) {
		$arStatuses[$arStatus["ID"]] =
			"[" . $arStatus["ID"] . "] " . $arStatus["NAME"];
		$statusesWithoutNoChange[$arStatus["ID"]] =
			"[" . $arStatus["ID"] . "] " . $arStatus["NAME"];
	}

	$orderStatuses = array_slice($arStatuses, 1);

	return $orderStatuses;
}

function GetPayMents()
{
	$arPaySystem = array();
	$resPaySystem = CSalePaySystem::GetList(
		$arOrder = array(
			"SORT" => "ASC",
			"NAME" => "ASC"
		)
	);

	while ($ptype = $resPaySystem->Fetch()) {
		$arPaySystem[$ptype["ID"]] = $ptype["NAME"];
	}

	return $arPaySystem;
}

if (!$USER->IsAdmin()) {
	return;
}

Loader::includeModule($moduleId);
Loader::includeModule('sale');
Loc::loadMessages(__FILE__);

$aTabs = array(
	array(
		'DIV' => $moduleId . '-options',
		"ICON" => "main_user_edit",
		'TAB' => GetMessage('MAIN_TAB_SET'),
		'TITLE' => GetMessage('MAIN_TAB_TITLE_SET')
	),
	array(
		'DIV' => $moduleId . '-receipt_type_val1',
		"ICON" => "main_settings",
		'TAB' => GetMessage('BIFIT_RECEIPT_TYPE_VAL1'),
		'TITLE' => GetMessage('BIFIT_RECEIPT_TYPE_VAL1')
	),
	array(
		'DIV' => $moduleId . '-receipt_type_val2',
		"ICON" => "main_settings",
		'TAB' => GetMessage('BIFIT_RECEIPT_TYPE_VAL2'),
		'TITLE' => GetMessage('BIFIT_RECEIPT_TYPE_VAL2')
	),
	array(
		'DIV' => $moduleId . '-receipt_type_val3',
		"ICON" => "main_settings",
		'TAB' => GetMessage('BIFIT_RECEIPT_TYPE_VAL3'),
		'TITLE' => GetMessage('BIFIT_RECEIPT_TYPE_VAL3')
	),
	array(
		'DIV' => $moduleId . '-receipt_type_val4',
		"ICON" => "main_settings",
		'TAB' => GetMessage('BIFIT_RECEIPT_TYPE_VAL4'),
		'TITLE' => GetMessage('BIFIT_RECEIPT_TYPE_VAL4')
	)
);

$form = new CAdminForm("tabControl", $aTabs);

$errors = [];

$f = $_REQUEST[$varname];
$s = bitrix_sessid();
$g = \Bitrix\Main\Context::getCurrent()
	->getRequest()
	->getHeader('X-Bitrix-Csrf-Token');
$h = bitrix_sessid();
$bitrixsessid = check_bitrix_sessid();

if ($REQUEST_METHOD == 'POST' && check_bitrix_sessid()) {
	$data = array(
		'test' => 'integer',
		'token' => 'string',
		'cashier_name' => 'string',
		'cashier_inn' => 'string',
		'payment_address' => 'string',
		'address_type' => 'string',
		'fio' => 'integer',
		'calculation_method' => 'string',
		'payment_type' => 'string',
		'vat_mode' => 'integer',
		'vat' => 'string',
		'use_type_1' => 'integer',
		'order_paid_1' => 'integer',
		'pm_statuses_1' => 'array',
		'pm_ids_1' => 'array',
		'tax_system_1' => 'string',
		'use_type_2' => 'integer',
		'order_paid_2' => 'integer',
		'pm_statuses_2' => 'array',
		'pm_ids_2' => 'array',
		'tax_system_2' => 'string',
		'use_type_3' => 'integer',
		'order_paid_3' => 'integer',
		'pm_statuses_3' => 'array',
		'pm_ids_3' => 'array',
		'tax_system_3' => 'string',
		'use_type_4' => 'integer',
		'order_paid_4' => 'integer',
		'pm_statuses_4' => 'array',
		'pm_ids_4' => 'array',
		'tax_system_4' => 'string'
	);

	// Todo обязательные поля
	foreach ($data as $key => $type) {
		$value = filter_input(INPUT_POST, strtoupper($key));

		if ($type == 'string') {
			COption::SetOptionString($moduleId, $key, $value);
		} else {
			if ($type == 'bool') {
				COption::SetOptionInt($moduleId, $key, $value === null ? 0 : 1);
			} else {
				if ($type == 'integer') {
					COption::SetOptionInt($moduleId, $key, $value);
				} else {
					if ($type == 'array') {
						$value = filter_input(
							INPUT_POST,
							strtoupper($key),
							FILTER_DEFAULT,
							FILTER_FORCE_ARRAY
						);
						COption::SetOptionString(
							$moduleId,
							$key,
							json_encode($value)
						);
					}
				}
			}
		}
	}

	CAdminMessage::showMessage(array(
		"MESSAGE" => GetMessage('BIFIT_OPTIONS_SAVED'),
		"TYPE" => "OK"
	));
}

$queryData = http_build_query(array(
	'lang' => LANGUAGE_ID,
	'mid' => $moduleId
));

$form->BeginEpilogContent();
echo bitrix_sessid_post();
$form->EndEpilogContent();

$form->Begin(array(
	'FORM_ACTION' => '/bitrix/admin/settings.php?' . $queryData
));
$form->BeginNextFormTab();

/* --- Settings --- */

$form->AddDropDownField(
	'TEST',
	GetMessage('BIFIT_TEST_LABEL'),
	true,
	array(
		1 => GetMessage('BIFIT_YES'),
		0 => GetMessage('BIFIT_NO')
	),
	COption::GetOptionString($moduleId, 'test')
);

$form->AddEditField(
	'TOKEN',
	GetMessage('BIFIT_TOKEN_LABEL'),
	true,
	array(
		'size' => 60,
		'maxlength' => 255
	),
	COption::GetOptionString($moduleId, 'token')
);

$form->AddEditField(
	'CASHIER_NAME',
	GetMessage('BIFIT_CASHIER_NAME_LABEL'),
	true,
	array(
		'size' => 60,
		'maxlength' => 255
	),
	COption::GetOptionString($moduleId, 'cashier_name')
);

$form->AddEditField(
	'CASHIER_INN',
	GetMessage('BIFIT_CASHIER_INN_LABEL'),
	true,
	array(
		'size' => 60,
		'maxlength' => 255
	),
	COption::GetOptionString($moduleId, 'cashier_inn')
);

$form->AddEditField(
	'PAYMENT_ADDRESS',
	GetMessage('BIFIT_PAYMENT_ADDRESS_LABEL'),
	true,
	array(
		'size' => 60,
		'maxlength' => 255
	),
	COption::GetOptionString($moduleId, 'payment_address')
);

$form->AddDropDownField(
	'ADDRESS_TYPE',
	GetMessage('BIFIT_ADDRESS_TYPE_LABELL'),
	true,
	array(
		'email' => GetMessage('BIFIT_EMAIL_LABEL'),
		'phone' => GetMessage('BIFIT_PHONE_LABEL')
	),
	COption::GetOptionString($moduleId, 'log')
);

$form->AddDropDownField(
	'FIO',
	GetMessage('BIFIT_FIO_LABEL'),
	true,
	array(
		1 => GetMessage('BIFIT_YES'),
		0 => GetMessage('BIFIT_NO')
	),
	COption::GetOptionString($moduleId, 'fio')
);

$form->AddDropDownField(
	'CALCULATION_METHOD',
	GetMessage('BIFIT_CALCULATION_METHOD_LABEL'),
	true,
	array(
		'PREPAY_FULL' => GetMessage('BIFIT_CALCULATION_METHOD_VAL1'),
		'PREPAY_PARTIAL' => GetMessage('BIFIT_CALCULATION_METHOD_VAL2'),
		'AVANS' => GetMessage('BIFIT_CALCULATION_METHOD_VAL3'),
		'FULL_PAY' => GetMessage('BIFIT_CALCULATION_METHOD_VAL4'),
		'PARTIAL_SETTLEMENT_AND_CREDIT' => GetMessage(
			'BIFIT_CALCULATION_METHOD_VAL5'
		),
		'TRANSFER_ON_CREDIT' => GetMessage('BIFIT_CALCULATION_METHOD_VAL6'),
		'CREDIT_PAYMENT' => GetMessage('BIFIT_CALCULATION_METHOD_VAL7')
	),
	COption::GetOptionString($moduleId, 'calculation_method')
);

$form->AddDropDownField(
	'PAYMENT_TYPE',
	GetMessage('BIFIT_PAYMENT_TYPE_LABEL'),
	true,
	array(
		'CASH' => GetMessage('BIFIT_PAYMENT_TYPE_VAL1'),
		'CARD' => GetMessage('BIFIT_PAYMENT_TYPE_VAL2'),
		'PREPAY' => GetMessage('BIFIT_PAYMENT_TYPE_VAL3'),
		'POSTPAY' => GetMessage('BIFIT_PAYMENT_TYPE_VAL4'),
		'OTHER' => GetMessage('BIFIT_PAYMENT_TYPE_VAL5')
	),
	COption::GetOptionString($moduleId, 'payment_type')
);

$form->AddDropDownField(
	'VAT_MODE',
	GetMessage('BIFIT_VAT_MODE_LABEL'),
	true,
	array(
		1 => GetMessage('BIFIT_YES'),
		0 => GetMessage('BIFIT_NO')
	),
	COption::GetOptionString($moduleId, 'vat_mode')
);

$form->AddDropDownField(
	'VAT',
	GetMessage('BIFIT_VAT_LABEL'),
	true,
	array(
		'WITHOUT_VAT' => GetMessage('BIFIT_VAT_VAL1'),
		'VAT_0' => GetMessage('BIFIT_VAT_VAL2'),
		'VAT_10' => GetMessage('BIFIT_VAT_VAL3'),
		'VAT_20' => GetMessage('BIFIT_VAT_VAL4'),
		'VAT_110' => GetMessage('BIFIT_VAT_VAL5'),
		'VAT_120' => GetMessage('BIFIT_VAT_VAL6')
	),
	COption::GetOptionString($moduleId, 'vat')
);

$orderStatuses = GetStatuses();
$arPaySystem = GetPayMents();
$arrTaxSystem = array(
	'COMMON' => GetMessage('BIFIT_TAX_SYSTEM_VAL1'),
	'SIMPLIFIED' => GetMessage('BIFIT_TAX_SYSTEM_VAL2'),
	'SIMPLIFIED_WITH_EXPENSE' => GetMessage('BIFIT_TAX_SYSTEM_VAL3'),
	'ENVD' => GetMessage('BIFIT_TAX_SYSTEM_VAL4'),
	'COMMON_AGRICULTURAL' => GetMessage('BIFIT_TAX_SYSTEM_VAL5'),
	'PATENT' => GetMessage('BIFIT_TAX_SYSTEM_VAL6')
);

/* ---/settings --- */

/* --- TAB #1 --- */

$form->BeginNextFormTab();
$form->AddDropDownField(
	'USE_TYPE_1',
	GetMessage('BIFIT_USE'),
	true,
	array(
		1 => GetMessage('BIFIT_YES'),
		0 => GetMessage('BIFIT_NO')
	),
	COption::GetOptionString($moduleId, 'use_type_1')
);
$form->AddDropDownField(
	'ORDER_PAID_1',
	GetMessage('BIFIT_ORDER_PAID'),
	true,
	array(
		2 => GetMessage('BIFIT_YES'),
		1 => GetMessage('BIFIT_NO'),
		0 => GetMessage('BIFIT_NOT_USE')
	),
	COption::GetOptionString($moduleId, 'order_paid_1')
);
$form->AddDropDownField(
	'TAX_SYSTEM_1',
	GetMessage('BIFIT_TAX_SYSTEM_LABEL'),
	true,
	$arrTaxSystem,
	COption::GetOptionString($moduleId, 'tax_system_1')
);

AddMultiSelectField(
	$form,
	'PM_STATUSES_1[]',
	GetMessage('BIFIT_STATUSES_LABEL'),
	true,
	$orderStatuses,
	json_decode(COption::GetOptionString($moduleId, 'pm_statuses_1'))
);

AddMultiSelectField(
	$form,
	'PM_IDS_1[]',
	GetMessage('BIFIT_PAYMENTS_LABEL'),
	true,
	$arPaySystem,
	json_decode(COption::GetOptionString($moduleId, 'pm_ids_1'))
);

/* ---/tab #1 --- */

/* --- TAB #2 --- */

$form->BeginNextFormTab();
$form->AddDropDownField(
	'USE_TYPE_2',
	GetMessage('BIFIT_USE'),
	true,
	array(
		1 => GetMessage('BIFIT_YES'),
		0 => GetMessage('BIFIT_NO')
	),
	COption::GetOptionString($moduleId, 'use_type_2')
);
$form->AddDropDownField(
	'ORDER_PAID_2',
	GetMessage('BIFIT_ORDER_PAID'),
	true,
	array(
		2 => GetMessage('BIFIT_YES'),
		1 => GetMessage('BIFIT_NO'),
		0 => GetMessage('BIFIT_NOT_USE')
	),
	COption::GetOptionString($moduleId, 'order_paid_2')
);
$form->AddDropDownField(
	'TAX_SYSTEM_2',
	GetMessage('BIFIT_TAX_SYSTEM_LABEL'),
	true,
	$arrTaxSystem,
	COption::GetOptionString($moduleId, 'tax_system_2')
);

AddMultiSelectField(
	$form,
	'PM_STATUSES_2[]',
	GetMessage('BIFIT_STATUSES_LABEL'),
	true,
	$orderStatuses,
	json_decode(COption::GetOptionString($moduleId, 'pm_statuses_2'))
);

AddMultiSelectField(
	$form,
	'PM_IDS_2[]',
	GetMessage('BIFIT_PAYMENTS_LABEL'),
	true,
	$arPaySystem,
	json_decode(COption::GetOptionString($moduleId, 'pm_ids_2'))
);

/* ---/tab #2 --- */

/* --- TAB #3 --- */

$form->BeginNextFormTab();
$form->AddDropDownField(
	'USE_TYPE_3',
	GetMessage('BIFIT_USE'),
	true,
	array(
		1 => GetMessage('BIFIT_YES'),
		0 => GetMessage('BIFIT_NO')
	),
	COption::GetOptionString($moduleId, 'use_type_3')
);
$form->AddDropDownField(
	'ORDER_PAID_3',
	GetMessage('BIFIT_ORDER_PAID'),
	true,
	array(
		2 => GetMessage('BIFIT_YES'),
		1 => GetMessage('BIFIT_NO'),
		0 => GetMessage('BIFIT_NOT_USE')
	),
	COption::GetOptionString($moduleId, 'order_paid_3')
);
$form->AddDropDownField(
	'TAX_SYSTEM_3',
	GetMessage('BIFIT_TAX_SYSTEM_LABEL'),
	true,
	$arrTaxSystem,
	COption::GetOptionString($moduleId, 'tax_system_3')
);

AddMultiSelectField(
	$form,
	'PM_STATUSES_3[]',
	GetMessage('BIFIT_STATUSES_LABEL'),
	true,
	$orderStatuses,
	json_decode(COption::GetOptionString($moduleId, 'pm_statuses_3'))
);

AddMultiSelectField(
	$form,
	'PM_IDS_3[]',
	GetMessage('BIFIT_PAYMENTS_LABEL'),
	true,
	$arPaySystem,
	json_decode(COption::GetOptionString($moduleId, 'pm_ids_3'))
);

/* ---/tab #3 --- */

/* --- TAB #4 --- */

$form->BeginNextFormTab();
$form->AddDropDownField(
	'USE_TYPE_4',
	GetMessage('BIFIT_USE'),
	true,
	array(
		1 => GetMessage('BIFIT_YES'),
		0 => GetMessage('BIFIT_NO')
	),
	COption::GetOptionString($moduleId, 'use_type_4')
);
$form->AddDropDownField(
	'ORDER_PAID_4',
	GetMessage('BIFIT_ORDER_PAID'),
	true,
	array(
		2 => GetMessage('BIFIT_YES'),
		1 => GetMessage('BIFIT_NO'),
		0 => GetMessage('BIFIT_NOT_USE')
	),
	COption::GetOptionString($moduleId, 'order_paid_4')
);
$form->AddDropDownField(
	'TAX_SYSTEM_4',
	GetMessage('BIFIT_TAX_SYSTEM_LABEL'),
	true,
	$arrTaxSystem,
	COption::GetOptionString($moduleId, 'tax_system_4')
);

AddMultiSelectField(
	$form,
	'PM_STATUSES_4[]',
	GetMessage('BIFIT_STATUSES_LABEL'),
	true,
	$orderStatuses,
	json_decode(COption::GetOptionString($moduleId, 'pm_statuses_4'))
);

AddMultiSelectField(
	$form,
	'PM_IDS_4[]',
	GetMessage('BIFIT_PAYMENTS_LABEL'),
	true,
	$arPaySystem,
	json_decode(COption::GetOptionString($moduleId, 'pm_ids_4'))
);

/* ---/tab #4 --- */

$form->Buttons(array(
	'disabled' => false,
	'back_url' => empty($back_url) ? 'settings.php?lang=' . LANG : $back_url
));

$form->Show();

/* --- Reports --- */

$list = new CAdminList('bifitkassa_reports');
$list->addHeaders(array(
	array(
		"id" => 'id',
		"content" => GetMessage('BIFIT_REPORTS_ITEM_ID'),
		"default" => true
	),
	array(
		"id" => 'order_id',
		"content" => GetMessage('BIFIT_REPORTS_ITEM_ORDER_ID'),
		"default" => true
	),
	array(
		"id" => 'order_number',
		"content" => GetMessage('BIFIT_NUMBER'),
		"default" => true
	),
	array(
		"id" => 'order_date',
		"content" => GetMessage('BIFIT_DATE'),
		"default" => true
	),
	array(
		"id" => 'cashier',
		"content" => GetMessage('BIFIT_CASHIER'),
		"default" => true
	),
	array(
		"id" => 'client',
		"content" => GetMessage('BIFIT_CLIENT'),
		"default" => true
	),
	array(
		"id" => 'receipt_link',
		"content" => GetMessage('BIFIT_RECEIPT'),
		"default" => true
	),
	array(
		"id" => 'doc',
		"content" => GetMessage('BIFIT_DOCUMENT'),
		"default" => true
	),
	array(
		"id" => 'log',
		"content" => GetMessage('BIFIT_LOG'),
		"default" => true
	),
	array(
		"id" => 'receipt_type',
		"content" => GetMessage('BIFIT_TYPE'),
		"default" => true
	),
	array(
		"id" => 'bifit_payment',
		"content" => GetMessage('BIFIT_PAYMENT'),
		"default" => true
	)
));

$page = filter_input(INPUT_GET, 'PAGEN_1', FILTER_VALIDATE_INT);
$page = $page ? $page : 1;
$limit = 10;
$offset = abs(intval($page * $limit - $limit));

try {
	$totalItems = (int) BifitKassaReportsTable::getCount();
} catch (\Bitrix\Main\DB\SqlException $e) {
	global $DB;

	$DB->RunSQLBatch(
		$_SERVER["DOCUMENT_ROOT"] .
			"/bitrix/modules/bifit.kassa/install/db/" .
			strtolower($DB->type) .
			"/uninstall.sql"
	);
	$DB->RunSQLBatch(
		$_SERVER["DOCUMENT_ROOT"] .
			"/bitrix/modules/bifit.kassa/install/db/" .
			strtolower($DB->type) .
			"/install.sql"
	);

	$totalItems = (int) BifitKassaReportsTable::getCount();
}

$items = BifitKassaReportsTable::getList(array(
	'select' => array('*'),
	'order' => array(
		'ID' => 'DESC'
	),
	'offset' => $offset,
	'limit' => $limit
));

while ($item = $items->fetch()) {
	$list->AddRow('rowid', array(
		'id' => $item['ID'],
		'order_id' => $item['ORDER_ID'],
		'order_number' => $item['ORDER_NUMBER'],
		'order_date' => $item['ORDER_DATE'],
		'cashier' => $item['CASHIER'],
		'client' => $item['CLIENT'],
		'receipt_link' => $item['RECEIPT_LINK'],
		'receipt_id' => $item['RECEIPT_ID'],
		'receipt_type' => $item['RECEIPT_TYPE'],
		'payment_type' => $item['PAYMENT_TYPE'],
		'doc' => $item['DOC'],
		'log' => $item['LOG']
	));
}

/* ---/reports --- */

$form->End();